import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";

local frame = 0;
local width = 163;

TargetPMoraleWindow = class(Turbine.UI.Window);
function TargetPMoraleWindow:Constructor()
	Turbine.UI.Window.Constructor( self );
	self:SetWantsUpdates(false);

	self.settings = TargetPMoraleWindowLoadSettings()

	self:SetSize(width, 9);
	self:SetPosition( self.settings.x, self.settings.y );
--	self:SetOpacity(1);
	self:SetBackground("Risko/TargetPMorale/images/targetmorale.tga");

	self.dragging = false;
	self.move = true;
	self.moraleFlushTime=0;
	self.prevmorale=100;
	
	self.player = Turbine.Gameplay.LocalPlayer.GetInstance();
	self.target = self.player:GetTarget();
	
	self.player.TargetChanged = function(sender, args)
		if self.player:GetTarget() == nil then
			if self.target ~= nil then
				self.target.MoraleChanged = nil
				self.target.MaxMoraleChanged = nil
				self.target.PowerChanged = nil
				self.target.MaxPowerChanged = nil
			end
			self.target = nil
			self:SetVisible(false); 
		else
			self.target = self.player:GetTarget();
			self.target.MoraleChanged = self.localMoraleChanged
			self.target.MaxMoraleChanged = self.localMaxMoraleChanged
			self.target.PowerChanged = self.localPowerChanged
			self.target.MaxPowerChanged = self.localMaxPowerChanged
			self:updateMorale()
			self:updatePower()
		end
	end

	self.moraleLabel=Turbine.UI.Label();
	self.moraleLabel:SetParent(self);
	self.moraleLabel:SetPosition( 0, 0 );
	self.moraleLabel:SetSize(0, 9 );
	self.moraleLabel:SetBackColor(Turbine.UI.Color(0,0,0));

	self.morale=Turbine.UI.Label();
	self.morale:SetParent(self);
	self.morale:SetFont(Turbine.UI.Lotro.Font.Verdana12);
	self.morale:SetText("" );
	self.morale:SetPosition( 0, -2 );
	self.morale:SetSize(width, 12);
	self.morale:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter);
	self.morale:SetFontStyle( Turbine.UI.FontStyle.Outline );
	
	self.morale.MouseDown = function(sender,args)
		self.MouseDown(self,args);
	end
	self.morale.MouseMove = function(sender,args)
		self.MouseMove(self,args);
	end
	self.morale.MouseUp = function(sender,args)
		self.MouseUp(self,args);
	end


	self.powerLabel=Turbine.UI.Label();
	self.powerLabel:SetParent(self);
	self.powerLabel:SetPosition( 0, 10 );
	self.powerLabel:SetSize(0, 9 );
	self.powerLabel:SetBackColor(Turbine.UI.Color(0,0,0));

	self.power=Turbine.UI.Label();
	self.power:SetParent(self);
	self.power:SetFont(Turbine.UI.Lotro.Font.Verdana12);
	self.power:SetText("" );
	self.power:SetPosition( 1, 9 );
	self.power:SetSize(width-1, 12);
	self.power:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter);
	self.power:SetFontStyle( Turbine.UI.FontStyle.Outline );
	
	self.morale.MouseDown = function(sender,args)
		self.MouseDown(self,args);
	end
	self.morale.MouseMove = function(sender,args)
		self.MouseMove(self,args);
	end
	self.morale.MouseUp = function(sender,args)
		self.MouseUp(self,args);
	end

	self.flashtimer = Turbine.Engine.GetGameTime();

	self.KeyDown = function(sender, args)
		if args.Action == 0x100000B3 then
			if self.visible == true then
				self:SetVisible(false); 
				self.visible = false;
			else
				self:SetVisible(true); 
				self.visible = true;
			end
		end
	end
	
	if self.settings.showPower then
		self:SetSize(width, 20);
	end

	self.visible=self.settings.show;
	self:SetVisible(self.vislble); 
	self:SetWantsUpdates(false);
	self:SetWantsKeyEvents(true);

	self.player.TargetChanged()
end

local frame = 0;



function TargetPMoraleWindow.localMoraleChanged(sender, args)
	targetPMoraleWindow:updateMorale()
end
function TargetPMoraleWindow.localMaxMoraleChanged(sender, args)
	targetPMoraleWindow:updateMorale()
end
function TargetPMoraleWindow.localPowerChanged(sender, args)
	targetPMoraleWindow:updatePower()
end
function TargetPMoraleWindow.localMaxPowerChanged(sender, args)
	targetPMoraleWindow:updatePower()
end


function TargetPMoraleWindow:updateMorale()
	if self.target == nil then return; end

	local flag, temp = pcall(self.target.GetMorale, self.target);
	if flag and temp ~= 0 and self.target:GetMaxMorale() ~= 0 then
		local morale=self.target:GetMorale();
		local maxmorale=self.target:GetMaxMorale();
		local p=morale/maxmorale;
		if p > 1 then p = 1 end
		local str = ""
		
		if self.settings.showAM then str= str..self:comma(morale); end
		if self.settings.showAMM then str= str.."/"..self:comma(maxmorale); end
		if self.settings.showPM then 
			local pstr= string.format("%.1f",math.floor(1000*p)/10);
			if pstr=="100.0" then
				pstr="100"
			end
			str = str..((str == "") and "" or " ").. pstr.."%";
		end

		self.morale:SetText(str);
		local w = math.floor(width*p)
		self.moraleLabel:SetSize( width-w, 9 );

		if self.visible == true then
			self:SetVisible(true); 
		else
			self:SetVisible(false); 
		end
	else
		self.morale:SetText("");
		self:SetVisible(false); 
	end
end

function TargetPMoraleWindow:updatePower()
	if self.target == nil then return; end

	local flag, temp = pcall(self.target.GetMorale, self.target);
	if flag and temp ~= 0 and self.target:GetMaxMorale() ~= 0 then
		local power=self.target:GetPower();
		local maxpower=self.target:GetMaxPower();
		local p=power/maxpower;
		if p > 1 then p = 1 end
		local str = ""
		
		if self.settings.showAP then str= str..self:comma(power); end
		if self.settings.showAMP then str= str.."/"..self:comma(maxpower); end
		if self.settings.showPP then 
			local pstr= string.format("%.1f",math.floor(1000*p)/10);
			if pstr=="100.0" then
				pstr="100"
			end
			str = str..((str == "") and "" or " ").. pstr.."%";
		end

		self.power:SetText(str);
		local w = math.floor(width*p)
		self.powerLabel:SetSize( width-w, 9 );

		if self.visible == true then
			self:SetVisible(true); 
		else
			self:SetVisible(false); 
		end
	else
		self.morale:SetText("");
		self:SetVisible(false); 
	end
end

function TargetPMoraleWindow:comma(n)
	if n > 1000000 then
		if self.settings.shrinkNum then
			s = string.format("%.1f", n/1000000).."M"
		else
			s = tostring(math.floor(n/1000000))..","..string.format("%03d",math.floor(n/1000)%1000)..","..string.format("%03d",math.floor(n%1000))
		end
	elseif n > 1000 then
		if self.settings.shrinkNum then
			s = string.format("%.1f", n/1000).."k"
		else
			s = tostring(math.floor(n/1000)%1000)..","..string.format("%03d",math.floor(n%1000));
		end
	else
		s = tostring(math.floor(n))
	end
	return tostring(s);
end

function TargetPMoraleWindow:Update(args)
--	frame = frame + 1;
--	self:showMorale();
end

function TargetPMoraleWindow:MouseDown(args)
	if args.Button == Turbine.UI.MouseButton.Left and self.move then
		self.dragStartX = args.X;
		self.dragStartY = args.Y;
		self.dragging = true;
	end
end

function TargetPMoraleWindow:MouseMove(args)

	if self.dragging then
		local x, y = self:GetPosition();
		x = x + (args.X - self.dragStartX);
		y = y + (args.Y - self.dragStartY);
		self.settings.x = x;
		self.settings.y = y;
		self:SetPosition(x, y);
	end
end

function TargetPMoraleWindow:MouseUp(args)
	if args.Button == Turbine.UI.MouseButton.Left then
		self:SaveSettings();
		self.dragging = false;
	end
end

function TargetPMoraleWindowLoadSettings()
	local settings = Turbine.PluginData.Load( Turbine.DataScope.Character, "TargetPMorale" );
	if type( settings ) ~= "table" then
		settings={ };
	end
	if (not settings.x) then settings.x = 400; end
	if (not settings.y) then settings.y = 400; end
	if (not settings.show) then settings.show = true; end
	if (not settings.showPM) then settings.showPM = true; end
	return settings;
end

function TargetPMoraleWindow:SaveSettings()
	Turbine.PluginData.Save( Turbine.DataScope.Character, "TargetPMorale", self.settings );
end

function TargetPMoraleWindow:show(flag)
	if flag then
		self.visible = true;
		self:SetVisible(true); 
		self.settings.show = true;
	else
		self.visible = false;
		self:SetVisible(false); 
		self.settings.show = false;
	end
	self:SaveSettings()
end


function TargetPMoraleWindow:shrinkNum(flag)
	self.settings.shrinkNum = flag;
	self:updateMorale()
	self:updatePower()
	self:SaveSettings()
end



function TargetPMoraleWindow:setAM(flag)
	self.settings.showAM = flag;
	self:updateMorale()
	self:SaveSettings()
end

function TargetPMoraleWindow:setAMM(flag)
	self.settings.showAMM = flag;
	self:updateMorale()
	self:SaveSettings()
end

function TargetPMoraleWindow:setPM(flag)
	self.settings.showPM = flag;
	self:updateMorale()
	self:SaveSettings()
end

function TargetPMoraleWindow:showPower(flag)
	if flag then
		self:SetSize(width, 20);
		self.settings.showPower = true;
	else
		self:SetSize(width, 9);
		self.settings.showPower = false;
	end
	self:updatePower()
	self:SaveSettings()
end

function TargetPMoraleWindow:setAP(flag)
	self.settings.showAP = flag;
	self:updatePower()
	self:SaveSettings()
end

function TargetPMoraleWindow:setAMP(flag)
	self.settings.showAMP = flag;
	self:updatePower()
	self:SaveSettings()
end

function TargetPMoraleWindow:setPP(flag)
	self.settings.showPP = flag;
	self:updatePower()
	self:SaveSettings()
end

